@@ -457,12 +457,9 @@ bool get_mysql_vars(MYSQL *connection)
 
   msg("Using server version %s", version_var);
 
-  if (opt_galera_info && !have_galera_enabled)
+  if (!(ret= detect_mysql_capabilities_for_backup()))
   {
-    msg("--galera-info is specified on the command "
-        "line, but the server does not support Galera "
-        "replication. Ignoring the option.");
-    opt_galera_info= false;
+    goto out;
   }
 
   /* make sure datadir value is the same in configuration file */
@@ -555,42 +552,42 @@ out:
   return (ret);
 }
 
+/*********************************************************************//**
+Query the server to find out what backup capabilities it supports.
+@return	true on success. */
+bool
+detect_mysql_capabilities_for_backup()
+{
+	/* do some sanity checks */
+	if (opt_galera_info && !have_galera_enabled) {
+		msg("--galera-info is specified on the command "
+		 	"line, but the server does not support Galera "
+		 	"replication. Ignoring the option.");
+		opt_galera_info = false;
+	}
+
+	if (opt_slave_info && have_multi_threaded_slave &&
+	    !have_gtid_slave) {
+	    	msg("The --slave-info option requires GTID enabled for a "
+			"multi-threaded slave.");
+		return(false);
+	}
+
+	return(true);
+}
+
 static
 bool
 select_incremental_lsn_from_history(lsn_t *incremental_lsn)
 {
 	MYSQL_RES *mysql_result;
 	char query[1000];
-	char buf[NAME_LEN*2+3];
-
-	size_t opt_incremental_history_name_len= 0;
-	size_t opt_incremental_history_uuid_len= 0;
-
-	if (opt_incremental_history_name)
-		opt_incremental_history_name_len=
-                  strlen(opt_incremental_history_name);
-
-	if (opt_incremental_history_uuid)
-		opt_incremental_history_uuid_len=
-                  strlen(opt_incremental_history_uuid);
-
-	if (opt_incremental_history_name_len*2 > sizeof(buf))
-		die("Incremental history table name '%s' is too long.",
-		    opt_incremental_history_name);
-
-	if (opt_incremental_history_uuid_len*2 > sizeof(buf))
-		die("Incremental history uuid '%s' is too long.",
-		    opt_incremental_history_uuid);
-
-	if (opt_incremental_history_name && opt_incremental_history_name[0]
-	    && opt_incremental_history_uuid && opt_incremental_history_uuid[0])
-		die("It is allowed to use either --incremental-history-name "
-		    "or --incremental-history-uuid, but not both.");
+	char buf[100];
 
 	if (opt_incremental_history_name) {
 		mysql_real_escape_string(mysql_connection, buf,
 				opt_incremental_history_name,
-				(unsigned long) opt_incremental_history_name_len);
+				(unsigned long)strlen(opt_incremental_history_name));
 		snprintf(query, sizeof(query),
 			"SELECT innodb_to_lsn "
 			"FROM " XB_HISTORY_TABLE " "
@@ -599,10 +596,11 @@ select_incremental_lsn_from_history(lsn_t *incremental_lsn)
 			"ORDER BY innodb_to_lsn DESC LIMIT 1",
 			buf);
 	}
-	else if (opt_incremental_history_uuid) {
+
+	if (opt_incremental_history_uuid) {
 		mysql_real_escape_string(mysql_connection, buf,
 				opt_incremental_history_uuid,
-				(unsigned long) opt_incremental_history_uuid_len);
+				(unsigned long)strlen(opt_incremental_history_uuid));
 		snprintf(query, sizeof(query),
 			"SELECT innodb_to_lsn "
 			"FROM " XB_HISTORY_TABLE " "
@@ -612,8 +610,6 @@ select_incremental_lsn_from_history(lsn_t *incremental_lsn)
 			buf);
 	}
 
-	/* xb_mysql_query(..,.., true) will die on error, so
-	mysql_result can't be nullptr */
 	mysql_result = xb_mysql_query(mysql_connection, query, true);
 
 	ut_ad(mysql_num_fields(mysql_result) == 1);
@@ -1415,7 +1411,7 @@ write_slave_info(ds_ctxt *datasink, MYSQL *connection)
   }
 
   mysql_slave_position= strdup(comment.c_ptr());
-  return datasink->backup_file_print_buf(XTRABACKUP_SLAVE_INFO,
+  return datasink->backup_file_print_buf(MB_SLAVE_INFO,
                                sql.ptr(), sql.length());
 }
 
@@ -1492,7 +1488,7 @@ write_galera_info(ds_ctxt *datasink, MYSQL *connection)
     goto cleanup;
   }
 
-  result= datasink->backup_file_printf(XTRABACKUP_GALERA_INFO,
+  result= datasink->backup_file_printf(MB_GALERA_INFO,
     "%s:%s %s\n", state_uuid ? state_uuid : state_uuid55,
     last_committed ? last_committed : last_committed55,
     domain_id ? domain_id : domain_id55);
@@ -1638,7 +1634,7 @@ write_binlog_info(ds_ctxt *datasink, MYSQL *connection)
 	read_mysql_variables(connection, "SHOW VARIABLES", vars, true);
 
 	if (filename == NULL || position == NULL) {
-		/* Do not create xtrabackup_binlog_info if binary
+		/* Do not create MB_BINLOG_INFO if binary
 		log is disabled */
 		result = true;
 		goto cleanup;
@@ -1654,14 +1650,14 @@ write_binlog_info(ds_ctxt *datasink, MYSQL *connection)
 			"filename '%s', position '%s', "
 			"GTID of the last change '%s'",
 			filename, position, gtid) != -1);
-		result = datasink->backup_file_printf(XTRABACKUP_BINLOG_INFO,
+		result = datasink->backup_file_printf(MB_BINLOG_INFO,
 					    "%s\t%s\t%s\n", filename, position,
 					    gtid);
 	} else {
 		ut_a(asprintf(&mysql_binlog_position,
 			"filename '%s', position '%s'",
 			filename, position) != -1);
-		result = datasink->backup_file_printf(XTRABACKUP_BINLOG_INFO,
+		result = datasink->backup_file_printf(MB_BINLOG_INFO,
 					    "%s\t%s\n", filename, position);
 	}
 
@@ -1696,7 +1692,7 @@ operator<<(std::ostream& s, const escape_and_quote& eq)
 }
 
 /*********************************************************************//**
-Writes xtrabackup_info file and if backup_history is enable creates
+Writes MB_INFO file and if backup_history is enable creates
 mysql.mariabackup_history and writes a new history record to the
 table containing all the history info particular to the just completed
 backup. */
@@ -1714,7 +1710,7 @@ write_xtrabackup_info(ds_ctxt *datasink,
 	char buf_end_time[100];
 	tm tm;
 	std::ostringstream oss;
-	const char *xb_stream_name[] = {"file", "mbstream"};
+	const char *xb_stream_name[] = {"file", "tar", "xbstream"};
 
 	uuid = read_mysql_one_value(connection, "SELECT UUID()");
 	server_version = read_mysql_one_value(connection, "SELECT VERSION()");
@@ -1773,7 +1769,7 @@ write_xtrabackup_info(ds_ctxt *datasink,
 		xb_stream_name[xtrabackup_stream_fmt], /* format */
 		xtrabackup_compress ? "compressed" : "N"); /* compressed */
 	if (buf_len < 0) {
-		msg("Error: cannot generate xtrabackup_info");
+		msg("Error: cannot generate " MB_INFO);
 		result = false;
 		goto cleanup;
 	}
@@ -1797,10 +1793,6 @@ write_xtrabackup_info(ds_ctxt *datasink,
 		goto cleanup;
 	}
 
-	xb_mysql_query(connection,
-		"ALTER TABLE IF EXISTS " XB_HISTORY_TABLE
-		" MODIFY format ENUM('file', 'tar', 'mbstream') DEFAULT NULL", false);
-
 	xb_mysql_query(connection,
 		"CREATE TABLE IF NOT EXISTS " XB_HISTORY_TABLE "("
 		"uuid VARCHAR(40) NOT NULL PRIMARY KEY,"
@@ -1818,7 +1810,7 @@ write_xtrabackup_info(ds_ctxt *datasink,
 		"innodb_to_lsn BIGINT UNSIGNED DEFAULT NULL,"
 		"partial ENUM('Y', 'N') DEFAULT NULL,"
 		"incremental ENUM('Y', 'N') DEFAULT NULL,"
-		"format ENUM('file', 'tar', 'mbstream') DEFAULT NULL,"
+		"format ENUM('file', 'tar', 'xbstream') DEFAULT NULL,"
 		"compressed ENUM('Y', 'N') DEFAULT NULL"
 		") CHARACTER SET utf8 ENGINE=innodb", false);
 
@@ -1922,7 +1914,7 @@ bool write_backup_config_file(ds_ctxt *datasink)
 		srv_log_file_size,
 		srv_page_size,
 		srv_undo_dir,
-		srv_undo_tablespaces,
+                (uint) srv_undo_tablespaces,
 		page_zip_level,
 		innobase_buffer_pool_filename ?
 			"innodb_buffer_pool_filename=" : "",
@@ -1969,7 +1961,7 @@ void
 capture_tool_command(int argc, char **argv)
 {
 	/* capture tool name tool args */
-	tool_name = strrchr(argv[0], IF_WIN('\\', '/'));
+	tool_name = strrchr(argv[0], '/');
 	tool_name = tool_name ? tool_name + 1 : argv[0];
 
 	make_argv(tool_args, sizeof(tool_args), argc, argv);
@@ -2068,7 +2060,6 @@ ulonglong get_current_lsn(MYSQL *connection)
 {
 	static const char lsn_prefix[] = "\nLog sequence number ";
 	ulonglong lsn = 0;
-	msg("Getting InnoDB LSN");
 	if (MYSQL_RES *res = xb_mysql_query(connection,
 					    "SHOW ENGINE INNODB STATUS",
 					    true, false)) {
@@ -2082,10 +2073,5 @@ ulonglong get_current_lsn(MYSQL *connection)
 		}
 		mysql_free_result(res);
 	}
-	msg("InnoDB LSN: %llu, Flushing Logs", lsn);
-        /* Make sure that current LSN is written and flushed to disk. */
-	xb_mysql_query(connection, "FLUSH NO_WRITE_TO_BINLOG ENGINE LOGS",
-		       false, false);
-	msg("Flushed Logs");
 	return lsn;
 }
