@@ -81,17 +81,16 @@ static int update_server_record_in_cache(FOREIGN_SERVER *existing,
 /* utility functions */
 static void merge_server_struct(FOREIGN_SERVER *from, FOREIGN_SERVER *to);
 
-static const uchar *servers_cache_get_key(const void *server_, size_t *length,
-                                          my_bool)
+static uchar *servers_cache_get_key(FOREIGN_SERVER *server, size_t *length,
+			       my_bool not_used __attribute__((unused)))
 {
-  auto server= static_cast<const FOREIGN_SERVER *>(server_);
   DBUG_ENTER("servers_cache_get_key");
   DBUG_PRINT("info", ("server_name_length %zd server_name %s",
                       server->server_name_length,
                       server->server_name));
 
-  *length= server->server_name_length;
-  DBUG_RETURN(reinterpret_cast<const uchar *>(server->server_name));
+  *length= (uint) server->server_name_length;
+  DBUG_RETURN((uchar*) server->server_name);
 }
 
 static PSI_memory_key key_memory_servers;
@@ -233,8 +232,8 @@ bool servers_init(bool dont_read_servers_table)
     DBUG_RETURN(TRUE);
 
   /* initialise our servers cache */
-  if (my_hash_init(key_memory_servers, &servers_cache, system_charset_info, 32,
-                   0, 0, servers_cache_get_key, 0, 0))
+  if (my_hash_init(key_memory_servers, &servers_cache, system_charset_info, 32, 0, 0,
+                   (my_hash_get_key) servers_cache_get_key, 0, 0))
   {
     return_val= TRUE; /* we failed, out of memory? */
     goto end;
@@ -253,6 +252,8 @@ bool servers_init(bool dont_read_servers_table)
   if (!(thd=new THD(0)))
     DBUG_RETURN(TRUE);
   thd->store_globals();
+  thd->set_query_inner((char*) STRING_WITH_LEN("intern:servers_init"),
+                       default_charset_info);
   /*
     It is safe to call servers_reload() since servers_* arrays and hashes which
     will be freed there are global static objects and thus are initialized
@@ -283,7 +284,7 @@ end:
 
 static bool servers_load(THD *thd, TABLE_LIST *tables)
 {
-  TABLE *table= tables[0].table;
+  TABLE *table;
   READ_RECORD read_record_info;
   bool return_val= TRUE;
   DBUG_ENTER("servers_load");
@@ -292,8 +293,7 @@ static bool servers_load(THD *thd, TABLE_LIST *tables)
   free_root(&mem, MYF(0));
   init_sql_alloc(key_memory_servers, &mem, ACL_ALLOC_BLOCK_SIZE, 0, MYF(0));
 
-  table->use_all_columns();
-  if (init_read_record(&read_record_info,thd,table, NULL, NULL,
+  if (init_read_record(&read_record_info,thd,table=tables[0].table, NULL, NULL,
                        1,0, FALSE))
     DBUG_RETURN(1);
   while (!(read_record_info.read_record()))
@@ -406,6 +406,7 @@ get_server_from_table_to_cache(TABLE *table)
   FOREIGN_SERVER *server= (FOREIGN_SERVER *)alloc_root(&mem,
                                                        sizeof(FOREIGN_SERVER));
   DBUG_ENTER("get_server_from_table_to_cache");
+  table->use_all_columns();
 
   /* get each field into the server struct ptr */
   ptr= get_field(&mem, table->field[0]);
@@ -462,7 +463,7 @@ get_server_from_table_to_cache(TABLE *table)
 
   RETURN VALUES
     0  - no error
-    other - ER_ error code
+    other - error code
 */
 
 static int 
@@ -546,19 +547,15 @@ insert_server_record_into_cache(FOREIGN_SERVER *server)
     advance of insertion into the mysql.servers table
 
   RETURN VALUE
-    0 - no errors
-    >0 - ER_ error code
+    VOID
+
 */
 
-static int
+static void 
 store_server_fields(TABLE *table, FOREIGN_SERVER *server)
 {
 
   table->use_all_columns();
-
-  if (table->s->fields < 9)
-    return ER_CANT_FIND_SYSTEM_REC;
-
   /*
     "server" has already been prepped by prepare_server_struct_for_<>
     so, all we need to do is check if the value is set (> -1 for port)
@@ -567,43 +564,30 @@ store_server_fields(TABLE *table, FOREIGN_SERVER *server)
     have changed will be set. If an insert, then all will be set,
     even if with empty strings
   */
-  if (server->host &&
+  if (server->host)
     table->field[1]->store(server->host,
-                           (uint) strlen(server->host), system_charset_info))
-    goto err;
-  if (server->db &&
+                           (uint) strlen(server->host), system_charset_info);
+  if (server->db)
     table->field[2]->store(server->db,
-                           (uint) strlen(server->db), system_charset_info))
-    goto err;
-  if (server->username &&
+                           (uint) strlen(server->db), system_charset_info);
+  if (server->username)
     table->field[3]->store(server->username,
-                           (uint) strlen(server->username), system_charset_info))
-    goto err;
-  if (server->password &&
+                           (uint) strlen(server->username), system_charset_info);
+  if (server->password)
     table->field[4]->store(server->password,
-                           (uint) strlen(server->password), system_charset_info))
-    goto err;
-  if (server->port > -1 &&
-    table->field[5]->store(server->port))
-    goto err;
-  if (server->socket &&
-    table->field[6]->store(server->socket,
-                           (uint) strlen(server->socket), system_charset_info))
-    goto err;
-  if (server->scheme &&
-    table->field[7]->store(server->scheme,
-                           (uint) strlen(server->scheme), system_charset_info))
-    goto err;
-  if (server->owner &&
-    table->field[8]->store(server->owner,
-                           (uint) strlen(server->owner), system_charset_info))
-    goto err;
-  return 0;
+                           (uint) strlen(server->password), system_charset_info);
+  if (server->port > -1)
+    table->field[5]->store(server->port);
 
-err:
-  THD *thd= table->in_use;
-  DBUG_ASSERT(thd->is_error());
-  return thd->get_stmt_da()->get_sql_errno();
+  if (server->socket)
+    table->field[6]->store(server->socket,
+                           (uint) strlen(server->socket), system_charset_info);
+  if (server->scheme)
+    table->field[7]->store(server->scheme,
+                           (uint) strlen(server->scheme), system_charset_info);
+  if (server->owner)
+    table->field[8]->store(server->owner,
+                           (uint) strlen(server->owner), system_charset_info);
 }
 
 /*
@@ -625,7 +609,7 @@ err:
 
   RETURN VALUE
     0 - no errors
-    >0 - ER_ error code
+    >0 - error code
 
   */
 
@@ -659,8 +643,7 @@ int insert_server_record(TABLE *table, FOREIGN_SERVER *server)
       error= 1;
     }
     /* store each field to be inserted */
-    if ((error= store_server_fields(table, server)))
-      DBUG_RETURN(error);
+    store_server_fields(table, server);
 
     DBUG_PRINT("info",("record for server '%s' not found!",
                        server->server_name));
@@ -972,7 +955,7 @@ void merge_server_struct(FOREIGN_SERVER *from, FOREIGN_SERVER *to)
     0, then index_read_idx is called to read the index to that record, the
     record then being ready to be updated, if found. If not found an error is
     set and error message printed. If the record is found, store_record is
-    called, then store_server_fields stores each field from the members of
+    called, then store_server_fields stores each field from the the members of
     the updated FOREIGN_SERVER struct.
 
   RETURN VALUE
@@ -990,15 +973,9 @@ update_server_record(TABLE *table, FOREIGN_SERVER *server)
 
   table->use_all_columns();
   /* set the field that's the PK to the value we're looking for */
-  if (table->field[0]->store(server->server_name,
+  table->field[0]->store(server->server_name,
                          server->server_name_length,
-                         system_charset_info))
-  {
-    DBUG_ASSERT(0); /* Protected by servers_cache */
-    THD *thd= table->in_use;
-    DBUG_ASSERT(thd->is_error());
-    return thd->get_stmt_da()->get_sql_errno();
-  }
+                         system_charset_info);
 
   if (unlikely((error=
                 table->file->ha_index_read_idx_map(table->record[0], 0,
@@ -1016,8 +993,7 @@ update_server_record(TABLE *table, FOREIGN_SERVER *server)
   {
     /* ok, so we can update since the record exists in the table */
     store_record(table,record[1]);
-    if ((error= store_server_fields(table, server)))
-      goto end;
+    store_server_fields(table, server);
     if (unlikely((error=table->file->ha_update_row(table->record[1],
                                                    table->record[0])) &&
                  error != HA_ERR_RECORD_IS_THE_SAME))
